@@ -20,21 +20,73 @@
 #ifndef PHP_IMAGICK_DEFS_H /* PHP_IMAGICK_DEFS_H */
 # define PHP_IMAGICK_DEFS_H
 
-/*
-	API exports
-*/
-#ifndef MY_IMAGICK_EXPORTS
-#  ifdef PHP_WIN32
-#    define MY_IMAGICK_EXPORTS __declspec(dllexport)
-#  else
-#    define MY_IMAGICK_EXPORTS PHPAPI
-#  endif
+/* Include magic wand header */
+#if defined (IM_MAGICKWAND_HEADER_STYLE_SEVEN)
+#  include <MagickWand/MagickWand.h>
+#elif defined (IM_MAGICKWAND_HEADER_STYLE_OLD)
+#  include <wand/magick-wand.h>
+#else
+#  include <wand/MagickWand.h>
 #endif
 
+#if MagickLibVersion >= 0x700
+#include "shim_im6_to_im7.h"
+#endif
+
+/* Some extra headers */
+#include "Zend/zend_exceptions.h"
+#include "Zend/zend_interfaces.h"
+#include "ext/standard/php_string.h"
+#include "ext/standard/info.h"
+#include "ext/standard/php_filestat.h"
+#include "php_ini.h"
+#include "Zend/zend.h"
+
+/* Include locale header */
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
+#if MagickLibVersion >= 0x680
+	#define IMAGICK_WITH_KERNEL
+#endif
+
+
+
+#if MagickLibVersion >= 0x700
+#if !defined(MaxTextExtent)
+# define MaxTextExtent  4096  /* always >= 4096 */
+#endif
+
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+	#define im_long zend_long
+#else
+	#define im_long long
+#endif
+
+
+typedef struct _php_imagick_callback {
+	void ***thread_ctx;
+#ifdef ZEND_ENGINE_3
+	zval user_callback;
+#else
+	zval *user_callback;
+#endif
+	struct _php_imagick_callback *previous_callback;
+} php_imagick_callback;
+
 /* Globals, needed for the ini settings */
 ZEND_BEGIN_MODULE_GLOBALS(imagick)
 	zend_bool locale_fix;
 	zend_bool progress_monitor;
+	zend_bool skip_version_check;
+	php_imagick_callback *progress_callback;
+#ifdef PHP_IMAGICK_ZEND_MM
+	MagickWand *keep_alive;
+#endif
 ZEND_END_MODULE_GLOBALS(imagick)
 
 #ifdef ZTS
@@ -53,6 +105,11 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 # endif
 #endif
 
+#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 3
+#define zend_parse_parameters_none() zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "")
+#endif
+
+
 /* Class names */
 #define PHP_IMAGICK_SC_NAME "Imagick"
 #define PHP_IMAGICK_EXCEPTION_SC_NAME "ImagickException"
@@ -66,80 +123,254 @@ ZEND_EXTERN_MODULE_GLOBALS(imagick)
 #define PHP_IMAGICKPIXELITERATOR_SC_NAME "ImagickPixelIterator"
 #define PHP_IMAGICKPIXELITERATOR_EXCEPTION_SC_NAME "ImagickPixelIteratorException"
 
+#ifdef IMAGICK_WITH_KERNEL
+#define PHP_IMAGICKKERNEL_SC_NAME "ImagickKernel"
+#define PHP_IMAGICKKERNEL_EXCEPTION_SC_NAME "ImagickKernelException"
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+
+/* Structure for Imagick object. */
+typedef struct _php_imagick_object  {
+	MagickWand *magick_wand;
+	char *progress_monitor_name;
+	zend_bool next_out_of_bound;
+	zend_object zo;
+} php_imagick_object;
+
+#else
+
 /* Structure for Imagick object. */
 typedef struct _php_imagick_object  {
 	zend_object zo;
 	MagickWand *magick_wand;
 	char *progress_monitor_name;
-	int next_out_of_bound;
+	zend_bool next_out_of_bound;
 } php_imagick_object;
 
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+
+/* Structure for ImagickDraw object. */
+typedef struct _php_imagickdraw_object  {
+	DrawingWand *drawing_wand;
+	zend_object zo;
+} php_imagickdraw_object;
+
+
+#else 
+
+
 /* Structure for ImagickDraw object. */
 typedef struct _php_imagickdraw_object  {
 	zend_object zo;
 	DrawingWand *drawing_wand;
 } php_imagickdraw_object;
 
+
+#endif
+
+#ifdef ZEND_ENGINE_3
+
 /* Structure for ImagickPixelIterator object. */
 typedef struct _php_imagickpixeliterator_object  {
-	zend_object zo;
+	
 	PixelIterator *pixel_iterator;
-	long instanciated_correctly;
-	int iterator_type;
+	zend_bool initialized;
+
 #if MagickLibVersion <= 0x628
 	long rows;
 	long iterator_position;
 #endif
+
+	zend_object zo;
 } php_imagickpixeliterator_object;
 
+#else
+
+/* Structure for ImagickPixelIterator object. */
+typedef struct _php_imagickpixeliterator_object  {
+	zend_object zo;
+	PixelIterator *pixel_iterator;
+	zend_bool initialized;
+
+#if MagickLibVersion <= 0x628
+	long rows;
+	long iterator_position;
+#endif
+
+} php_imagickpixeliterator_object;
+
+#endif
+
+
+#ifdef ZEND_ENGINE_3
+
 /* Structure for ImagickPixel object. */
 typedef struct _php_imagickpixel_object  {
-    zend_object zo;
-    PixelWand *pixel_wand;
-	int initialized_via_iterator;
+	PixelWand *pixel_wand;
+	zend_bool initialized_via_iterator;
+	zend_object zo;
 } php_imagickpixel_object;
 
+
+#else
+
+
+/* Structure for ImagickPixel object. */
+typedef struct _php_imagickpixel_object  {
+	zend_object zo;
+	PixelWand *pixel_wand;
+	zend_bool initialized_via_iterator;
+} php_imagickpixel_object;
+
+#endif
+
+
+#ifdef IMAGICK_WITH_KERNEL
+#ifdef ZEND_ENGINE_3
+/* Structure for ImagickKernel object. */
+typedef struct _php_imagickkernel_object  {
+	KernelInfo *kernel_info;
+	zend_object zo;
+} php_imagickkernel_object;
+#else
+
+/* Structure for ImagickKernel object. */
+typedef struct _php_imagickkernel_object  {
+    zend_object zo;
+    KernelInfo *kernel_info;
+} php_imagickkernel_object;
+#endif
+#endif
+
+//Object fetching.
+#ifdef ZEND_ENGINE_3 
+
+static inline php_imagick_object *php_imagick_fetch_object(zend_object *obj) {
+	return (php_imagick_object *)((char*)(obj) - XtOffsetOf(php_imagick_object, zo));
+}
+
+static inline php_imagickdraw_object *php_imagickdraw_fetch_object(zend_object *obj) {
+	return (php_imagickdraw_object *)((char*)(obj) - XtOffsetOf(php_imagickdraw_object, zo));
+}
+
+static inline php_imagickpixel_object *php_imagickpixel_fetch_object(zend_object *obj) {
+	return (php_imagickpixel_object *)((char*)(obj) - XtOffsetOf(php_imagickpixel_object, zo));
+}
+
+static inline php_imagickpixeliterator_object *php_imagickpixeliterator_fetch_object(zend_object *obj) {
+	return (php_imagickpixeliterator_object *)((char*)(obj) - XtOffsetOf(php_imagickpixeliterator_object, zo));
+}
+
+#ifdef IMAGICK_WITH_KERNEL
+static inline php_imagickkernel_object *php_imagickkernel_fetch_object(zend_object *obj) {
+	return (php_imagickkernel_object *)((char*)(obj) - XtOffsetOf(php_imagickkernel_object, zo));
+}
+#endif
+
+#else 
+	#define php_imagick_fetch_object(object) ((php_imagick_object *)object)
+	#define php_imagickdraw_fetch_object(object) ((php_imagickdraw_object *)object) 
+	#define php_imagickpixel_fetch_object(object) ((php_imagickpixel_object *)object)
+	#define php_imagickpixeliterator_fetch_object(object) ((php_imagickpixeliterator_object *)object)
+	#define php_imagickkernel_fetch_object(object) ((php_imagickkernel_object *)object)
+#endif
+
+// Object access
+#ifdef ZEND_ENGINE_3
+	#define Z_IMAGICK_P(zv) php_imagick_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKDRAW_P(zv) php_imagickdraw_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKPIXEL_P(zv) php_imagickpixel_fetch_object(Z_OBJ_P((zv)))
+	#define Z_IMAGICKPIXELITERATOR_P(zv) php_imagickpixeliterator_fetch_object(Z_OBJ_P((zv)))
+#ifdef IMAGICK_WITH_KERNEL
+	#define Z_IMAGICKKERNEL_P(zv) php_imagickkernel_fetch_object(Z_OBJ_P((zv)))
+#endif
+#else
+	#define Z_IMAGICK_P(zv) (php_imagick_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKDRAW_P(zv) (php_imagickdraw_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKPIXEL_P(zv) (php_imagickpixel_object *) zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKPIXELITERATOR_P(zv) (php_imagickpixeliterator_object *)zend_object_store_get_object(zv TSRMLS_CC)
+	#define Z_IMAGICKKERNEL_P(zv) (php_imagickkernel_object *)zend_object_store_get_object(zv TSRMLS_CC)
+#endif
+
+// String access
+#ifdef ZEND_ENGINE_3
+	#define IM_ZVAL_STRING(zv, charstar) ZVAL_STRING(zv, charstar);
+	#define IM_RETURN_STRING(s) RETURN_STRING(s)
+#else
+	#define IM_ZVAL_STRING(zv, charstar) ZVAL_STRING(zv, charstar, 1);
+	//RETURN_STRING(s, duplicate) 
+	#define IM_RETURN_STRING(s) RETURN_STRING(s, 0)
+#endif
+
+#ifdef ZEND_ENGINE_3
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr)
+	#define IM_ZVAL_STRINGL(zv, charstr, length) ZVAL_STRINGL(zv, charstr, length)
+	#define IM_add_next_index_string(zv, charstr) add_next_index_string(zv, charstr)
+	#define IM_LEN_TYPE size_t
+#else
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr, 1)
+	#define IM_add_assoc_string(zv, key, charstr) add_assoc_string(zv, key, charstr, 1)
+	#define IM_ZVAL_STRINGL(zv, charstr, length) ZVAL_STRINGL(zv, charstr, length, 1)
+	#define IM_add_next_index_string(zv, charstr) add_next_index_string(zv, charstr, 1)
+	#define IM_add_next_index_zval(zv1, zv2)
+	#define IM_LEN_TYPE int
+#endif
+
+#ifdef ZEND_ENGINE_3
+	#define IM_ZEND_OBJECT zend_object
+#else 
+	#define IM_ZEND_OBJECT void
+#endif
+
+
+
+
+
 /* Define some color constants */
-#define IMAGICKCOLORBLACK 11
-#define IMAGICKCOLORBLUE 12
-#define IMAGICKCOLORCYAN 13
-#define IMAGICKCOLORGREEN 14
-#define IMAGICKCOLORRED 15
-#define IMAGICKCOLORYELLOW 16
-#define IMAGICKCOLORMAGENTA 17
-#define IMAGICKCOLOROPACITY 18
-#define IMAGICKCOLORALPHA 19
-#define IMAGICKCOLORFUZZ 20
+typedef enum _php_imagick_color_t {
+	PHP_IMAGICK_COLOR_MIN = 10,
+	PHP_IMAGICK_COLOR_BLACK,
+	PHP_IMAGICK_COLOR_BLUE,
+	PHP_IMAGICK_COLOR_CYAN,
+	PHP_IMAGICK_COLOR_GREEN,
+	PHP_IMAGICK_COLOR_RED,
+	PHP_IMAGICK_COLOR_YELLOW,
+	PHP_IMAGICK_COLOR_MAGENTA,
+#if MagickLibVersion < 0x700
+	PHP_IMAGICK_COLOR_OPACITY,
+#endif
+	PHP_IMAGICK_COLOR_ALPHA,
+	PHP_IMAGICK_COLOR_FUZZ,
+	PHP_IMAGICK_COLOR_MAX,
+} php_imagick_color_t;
 
 /* Class enum */
-#define IMAGICK_CLASS 1
-#define IMAGICKDRAW_CLASS 2
-#define IMAGICKPIXELITERATOR_CLASS 3
-#define IMAGICKPIXEL_CLASS 4
-
-/* Free filename constants */
-#define IMAGICK_DONT_FREE_FILENAME 0
-#define IMAGICK_FREE_FILENAME 1
+typedef enum _php_imagick_class_type_t {
+	IMAGICK_CLASS,
+	IMAGICKDRAW_CLASS,
+	IMAGICKPIXELITERATOR_CLASS,
+	IMAGICKPIXEL_CLASS,
+#ifdef IMAGICK_WITH_KERNEL
+	IMAGICKKERNEL_CLASS
+#endif
+} php_imagick_class_type_t;
 
 /* Read / write constants */
-#define IMAGICK_READ_WRITE_NO_ERROR 0
-#define IMAGICK_READ_WRITE_SAFE_MODE_ERROR 1
-#define IMAGICK_READ_WRITE_OPEN_BASEDIR_ERROR 2
-#define IMAGICK_READ_WRITE_UNDERLYING_LIBRARY 3
-#define IMAGICK_READ_WRITE_PERMISSION_DENIED 4
-#define IMAGICK_READ_WRITE_FILENAME_TOO_LONG 5
-#define IMAGICK_READ_WRITE_PATH_DOES_NOT_EXIST 6
-
-/* Filehandle handling */
-#define IMAGICK_WRITE_IMAGE_FILE 1
-#define IMAGICK_WRITE_IMAGES_FILE 2
-#define IMAGICK_READ_IMAGE_FILE 3
-#define IMAGICK_PING_IMAGE_FILE 4
-
-/* Filename */
-#define PHP_IMAGICK_FILE_PLAIN	1
-#define PHP_IMAGICK_FILE_FORMAT	2
-#define PHP_IMAGICK_FILE_URL	3
+typedef enum _php_imagick_rw_result_t {
+	IMAGICK_RW_OK,
+	IMAGICK_RW_SAFE_MODE_ERROR,
+	IMAGICK_RW_OPEN_BASEDIR_ERROR,
+	IMAGICK_RW_UNDERLYING_LIBRARY,
+	IMAGICK_RW_PERMISSION_DENIED,
+	IMAGICK_RW_FILENAME_TOO_LONG,
+	IMAGICK_RW_PATH_DOES_NOT_EXIST,
+	IMAGICK_RW_PATH_IS_DIR,
+} php_imagick_rw_result_t;
 
 /* Class entries */
 extern zend_class_entry *php_imagick_sc_entry;
@@ -150,6 +381,10 @@ extern zend_class_entry *php_imagickpixel_sc_entry;
 extern zend_class_entry *php_imagickpixel_exception_class_entry;
 extern zend_class_entry *php_imagickpixeliterator_sc_entry;
 extern zend_class_entry *php_imagickpixeliterator_exception_class_entry;
+#ifdef IMAGICK_WITH_KERNEL
+extern zend_class_entry *php_imagickkernel_sc_entry;
+extern zend_class_entry *php_imagickkernel_exception_class_entry;
+#endif
 
 /* Forward declarations (Imagick) */
 
@@ -165,7 +400,9 @@ PHP_METHOD(imagick, vignetteimage);
 PHP_METHOD(imagick, compareimagelayers);
 PHP_METHOD(imagick, optimizeimagelayers);
 PHP_METHOD(imagick, uniqueimagecolors);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagematte);
+#endif
 PHP_METHOD(imagick, setimagematte);
 PHP_METHOD(imagick, adaptiveresizeimage);
 PHP_METHOD(imagick, sketchimage);
@@ -176,19 +413,25 @@ PHP_METHOD(imagick, adaptiveblurimage);
 PHP_METHOD(imagick, contraststretchimage);
 PHP_METHOD(imagick, adaptivesharpenimage);
 PHP_METHOD(imagick, randomthresholdimage);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, roundcornersimage);
+#endif
 PHP_METHOD(imagick, setiteratorindex);
 PHP_METHOD(imagick, getiteratorindex);
 PHP_METHOD(imagick, transformimage);
 #endif
 #if MagickLibVersion > 0x630
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageopacity);
+#endif
 PHP_METHOD(imagick, orderedposterizeimage);
 #endif
 #if MagickLibVersion > 0x631
 PHP_METHOD(imagick, polaroidimage);
 PHP_METHOD(imagick, getimageproperty);
 PHP_METHOD(imagick, setimageproperty);
+PHP_METHOD(imagick, deleteimageproperty);
+PHP_METHOD(imagick, identifyformat);
 PHP_METHOD(imagick, setimageinterpolatemethod);
 PHP_METHOD(imagick, getimageinterpolatemethod);
 PHP_METHOD(imagick, linearstretchimage);
@@ -199,9 +442,11 @@ PHP_METHOD(imagick, extentimage);
 PHP_METHOD(imagick, getimageorientation);
 PHP_METHOD(imagick, setimageorientation);
 #endif
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 #if MagickLibVersion > 0x634
 PHP_METHOD(imagick, paintfloodfillimage);
 #endif
+#endif
 #if MagickLibVersion > 0x635
 PHP_METHOD(imagick, clutimage);
 PHP_METHOD(imagick, getimageproperties);
@@ -210,11 +455,18 @@ PHP_METHOD(imagick, distortimage);
 PHP_METHOD(imagick, writeimagefile);
 PHP_METHOD(imagick, writeimagesfile);
 PHP_METHOD(imagick, resetimagepage);
+
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimageclipmask);
 PHP_METHOD(imagick, getimageclipmask);
+#endif
 PHP_METHOD(imagick, animateimages);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, recolorimage);
 #endif
+#endif
+#endif
 #if MagickLibVersion > 0x636
 PHP_METHOD(imagick, setfont);
 PHP_METHOD(imagick, getfont);
@@ -266,15 +518,25 @@ PHP_METHOD(imagick, transformimagecolorspace);
 #if MagickLibVersion > 0x652
 PHP_METHOD(imagick, haldclutimage);
 #endif
+#if MagickLibVersion > 0x655
+PHP_METHOD(imagick, autolevelimage);
+PHP_METHOD(imagick, blueshiftimage);
+#endif
 #if MagickLibVersion > 0x656
 PHP_METHOD(imagick, setimageartifact);
 PHP_METHOD(imagick, getimageartifact);
 PHP_METHOD(imagick, deleteimageartifact);
 PHP_METHOD(imagick, setcolorspace);
 PHP_METHOD(imagick, getcolorspace);
+PHP_METHOD(imagick, clampimage);
 #endif
+#if MagickLibVersion > 0x667
+PHP_METHOD(imagick, smushimages);
+#endif
+
 PHP_METHOD(imagick, __construct);
 PHP_METHOD(imagick, __tostring);
+PHP_METHOD(imagick, count);
 PHP_METHOD(imagick, getpixeliterator);
 PHP_METHOD(imagick, getpixelregioniterator);
 PHP_METHOD(imagick, readimage);
@@ -297,14 +559,18 @@ PHP_METHOD(imagick, previousimage);
 PHP_METHOD(imagick, nextimage);
 PHP_METHOD(imagick, haspreviousimage);
 PHP_METHOD(imagick, hasnextimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, setimageindex);
 PHP_METHOD(imagick, getimageindex);
+#endif
 PHP_METHOD(imagick, setimageformat);
 PHP_METHOD(imagick, setimagefilename);
 PHP_METHOD(imagick, getimagefilename);
 PHP_METHOD(imagick, getimageformat);
 PHP_METHOD(imagick, getimagemimetype);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagesize);
+#endif
 PHP_METHOD(imagick, getimagegeometry);
 PHP_METHOD(imagick, getimagecolors);
 PHP_METHOD(imagick, scaleimage);
@@ -321,10 +587,13 @@ PHP_METHOD(imagick, getimageblob);
 PHP_METHOD(imagick, getimagesblob);
 PHP_METHOD(imagick, setimagecompressionquality);
 PHP_METHOD(imagick, getimagecompressionquality);
+PHP_METHOD(imagick, setimagecompression);
+PHP_METHOD(imagick, getimagecompression);
 PHP_METHOD(imagick, annotateimage);
 PHP_METHOD(imagick, compositeimage);
 PHP_METHOD(imagick, modulateimage);
 PHP_METHOD(imagick, montageimage);
+PHP_METHOD(imagick, morphology);
 PHP_METHOD(imagick, identifyimage);
 PHP_METHOD(imagick, thresholdimage);
 PHP_METHOD(imagick, adaptivethresholdimage);
@@ -335,7 +604,9 @@ PHP_METHOD(imagick, charcoalimage);
 PHP_METHOD(imagick, normalizeimage);
 PHP_METHOD(imagick, oilpaintimage);
 PHP_METHOD(imagick, posterizeimage);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, radialblurimage);
+#endif
 PHP_METHOD(imagick, raiseimage);
 PHP_METHOD(imagick, resampleimage);
 PHP_METHOD(imagick, resizeimage);
@@ -344,8 +615,13 @@ PHP_METHOD(imagick, rotateimage);
 PHP_METHOD(imagick, sampleimage);
 PHP_METHOD(imagick, solarizeimage);
 PHP_METHOD(imagick, shadowimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, setimageattribute);
+#endif
 PHP_METHOD(imagick, setimagebackgroundcolor);
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagick, setimagechannelmask);
+#endif
 PHP_METHOD(imagick, setimagecompose);
 PHP_METHOD(imagick, setimagecompression);
 PHP_METHOD(imagick, setimagedelay);
@@ -355,6 +631,9 @@ PHP_METHOD(imagick, setimageiterations);
 PHP_METHOD(imagick, setimagemattecolor);
 PHP_METHOD(imagick, setimagepage);
 PHP_METHOD(imagick, setimageprogressmonitor);
+#if MagickLibVersion > 0x653
+PHP_METHOD(imagick, setprogressmonitor);
+#endif
 PHP_METHOD(imagick, setimageresolution);
 PHP_METHOD(imagick, setimagescene);
 PHP_METHOD(imagick, setimagetickspersecond);
@@ -372,18 +651,29 @@ PHP_METHOD(imagick, queryfonts);
 PHP_METHOD(imagick, queryfontmetrics);
 PHP_METHOD(imagick, steganoimage);
 PHP_METHOD(imagick, motionblurimage);
+#if MagickLibVersion < 0x700
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, mosaicimages);
+#endif
+#endif
 PHP_METHOD(imagick, morphimages);
 PHP_METHOD(imagick, minifyimage);
 PHP_METHOD(imagick, addnoiseimage);
 PHP_METHOD(imagick, affinetransformimage);
+#if MagickLibVersion < 0x700
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, averageimages);
+#endif
+#endif
 PHP_METHOD(imagick, borderimage);
+PHP_METHOD(imagick, calculatecrop);
 PHP_METHOD(imagick, chopimage);
 PHP_METHOD(imagick, clipimage);
 PHP_METHOD(imagick, clippathimage);
 PHP_METHOD(imagick, coalesceimages);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, colorfloodfillimage);
+#endif
 PHP_METHOD(imagick, colorizeimage);
 PHP_METHOD(imagick, compareimagechannels);
 PHP_METHOD(imagick, compareimages);
@@ -398,20 +688,36 @@ PHP_METHOD(imagick, embossimage);
 PHP_METHOD(imagick, enhanceimage);
 PHP_METHOD(imagick, equalizeimage);
 PHP_METHOD(imagick, evaluateimage);
+#if MagickLibVersion >= 0x687
+PHP_METHOD(imagick, evaluateimages);
+#endif
+#if MagickLibVersion < 0x700
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, flattenimages);
+#endif
+#endif
 PHP_METHOD(imagick, flipimage);
 PHP_METHOD(imagick, flopimage);
+#if MagickLibVersion >= 0x655
+PHP_METHOD(imagick, forwardfouriertransformimage);
+#endif
 PHP_METHOD(imagick, frameimage);
 PHP_METHOD(imagick, fximage);
 PHP_METHOD(imagick, gammaimage);
 PHP_METHOD(imagick, gaussianblurimage);
+#if MagickLibVersion < 0x700
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimageattribute);
+#endif
+#endif
 PHP_METHOD(imagick, getimagebackgroundcolor);
 PHP_METHOD(imagick, getimageblueprimary);
 PHP_METHOD(imagick, getimagebordercolor);
 PHP_METHOD(imagick, getimagechanneldepth);
 PHP_METHOD(imagick, getimagechanneldistortion);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimagechannelextrema);
+#endif
 PHP_METHOD(imagick, getimagechannelmean);
 PHP_METHOD(imagick, getimagechannelstatistics);
 PHP_METHOD(imagick, getimagecolormapcolor);
@@ -420,7 +726,9 @@ PHP_METHOD(imagick, getimagecompose);
 PHP_METHOD(imagick, getimagedelay);
 PHP_METHOD(imagick, getimagedepth);
 PHP_METHOD(imagick, getimagedistortion);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, getimageextrema);
+#endif
 PHP_METHOD(imagick, getimagedispose);
 PHP_METHOD(imagick, getimagegamma);
 PHP_METHOD(imagick, getimagegreenprimary);
@@ -447,23 +755,39 @@ PHP_METHOD(imagick, getnumberimages);
 PHP_METHOD(imagick, getimagetotalinkdensity);
 PHP_METHOD(imagick, getimageregion);
 PHP_METHOD(imagick, implodeimage);
+#if MagickLibVersion >= 0x658
+PHP_METHOD(imagick, inversefouriertransformimage);
+#endif
 PHP_METHOD(imagick, levelimage);
 PHP_METHOD(imagick, magnifyimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, mapimage);
 PHP_METHOD(imagick, mattefloodfillimage);
+#endif
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, medianfilterimage);
+#endif
 PHP_METHOD(imagick, negateimage);
+#if !defined(MAGICKCORE_EXCLUDE_DEPRECATED)
 PHP_METHOD(imagick, paintopaqueimage);
 PHP_METHOD(imagick, painttransparentimage);
+#endif
 PHP_METHOD(imagick, previewimages);
 PHP_METHOD(imagick, profileimage);
 PHP_METHOD(imagick, quantizeimage);
 PHP_METHOD(imagick, quantizeimages);
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, reducenoiseimage);
+#endif
 PHP_METHOD(imagick, removeimageprofile);
 PHP_METHOD(imagick, separateimagechannel);
 PHP_METHOD(imagick, sepiatoneimage);
+
+#if MagickLibVersion < 0x700
 PHP_METHOD(imagick, setimagebias);
+PHP_METHOD(imagick, setimagebiasquantum);
+#endif
+
 PHP_METHOD(imagick, setimageblueprimary);
 PHP_METHOD(imagick, setimagebordercolor);
 PHP_METHOD(imagick, setimagechanneldepth);
@@ -491,6 +815,10 @@ PHP_METHOD(imagick, newpseudoimage);
 PHP_METHOD(imagick, getcompression);
 PHP_METHOD(imagick, getcompressionquality);
 PHP_METHOD(imagick, getcopyright);
+PHP_METHOD(imagick, getconfigureoptions);
+#if MagickLibVersion > 0x660
+PHP_METHOD(imagick, getfeatures);
+#endif
 PHP_METHOD(imagick, getfilename);
 PHP_METHOD(imagick, getformat);
 PHP_METHOD(imagick, gethomeurl);
@@ -498,6 +826,8 @@ PHP_METHOD(imagick, getinterlacescheme);
 PHP_METHOD(imagick, getoption);
 PHP_METHOD(imagick, getpackagename);
 PHP_METHOD(imagick, getpage);
+PHP_METHOD(imagick, gethdrienabled);
+PHP_METHOD(imagick, getquantum);
 PHP_METHOD(imagick, getquantumdepth);
 PHP_METHOD(imagick, getquantumrange);
 PHP_METHOD(imagick, getreleasedate);
@@ -519,6 +849,42 @@ PHP_METHOD(imagick, setresolution);
 PHP_METHOD(imagick, setsamplingfactors);
 PHP_METHOD(imagick, setsize);
 PHP_METHOD(imagick, settype);
+PHP_METHOD(imagick, brightnesscontrastimage);
+#if MagickLibVersion > 0x661
+PHP_METHOD(imagick, colormatriximage);
+#endif
+PHP_METHOD(imagick, selectiveblurimage);
+#if MagickLibVersion >= 0x689
+PHP_METHOD(imagick, rotationalblurimage);
+#endif
+#if MagickLibVersion >= 0x683
+PHP_METHOD(imagick, statisticimage);
+#endif
+#if MagickLibVersion >= 0x652
+PHP_METHOD(imagick, subimagematch);
+#endif
+
+PHP_METHOD(imagick, setregistry);
+PHP_METHOD(imagick, getregistry);
+PHP_METHOD(imagick, listregistry);
+PHP_METHOD(imagick, morphology);
+PHP_METHOD(imagick, filter);
+PHP_METHOD(imagick, setantialias);
+PHP_METHOD(imagick, getantialias);
+#if MagickLibVersion > 0x676
+PHP_METHOD(imagick, colordecisionlistimage);
+#endif
+
+#if MagickLibVersion >= 0x660
+PHP_METHOD(imagick, autogammaimage);
+#endif //#if MagickLibVersion >= 0x660
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagick, autoorient);
+PHP_METHOD(imagick, compositeimagegravity);
+#endif // #if MagickLibVersion >= 0x692
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagick, localcontrastimage);
+#endif // #if MagickLibVersion >= 0x692
 
 /* Forward declarations (ImagickDraw) */
 #if MagickLibVersion > 0x628
@@ -566,7 +932,13 @@ PHP_METHOD(imagickdraw, translate);
 PHP_METHOD(imagickdraw, line);
 PHP_METHOD(imagickdraw, polygon);
 PHP_METHOD(imagickdraw, arc);
+
+#if MagickLibVersion >= 0x700
+PHP_METHOD(imagickdraw, alpha);
+#else
 PHP_METHOD(imagickdraw, matte);
+#endif
+
 PHP_METHOD(imagickdraw, point);
 PHP_METHOD(imagickdraw, getfontstretch);
 PHP_METHOD(imagickdraw, setfontstretch);
@@ -651,11 +1023,34 @@ PHP_METHOD(imagickdraw, setstrokeopacity);
 PHP_METHOD(imagickdraw, setvectorgraphics);
 PHP_METHOD(imagickdraw, pop);
 PHP_METHOD(imagickdraw, push);
+#if MagickLibVersion >= 0x693
+PHP_METHOD(imagickdraw, getopacity);
+PHP_METHOD(imagickdraw, setopacity);
+#endif //MagickLibVersion >= 0x693
+
+#if MagickLibVersion >= 0x675
+PHP_METHOD(imagickdraw, getfontresolution);
+PHP_METHOD(imagickdraw, setfontresolution);
+PHP_METHOD(imagickdraw, getbordercolor);
+PHP_METHOD(imagickdraw, setbordercolor);
+#endif //#if MagickLibVersion >= 0x675
+
+#if MagickLibVersion >= 0x693
+PHP_METHOD(imagickdraw, setdensity);
+PHP_METHOD(imagickdraw, getdensity);
+#endif// #if MagickLibVersion >= 0x693
+
+#if MagickLibVersion >= 0x692
+PHP_METHOD(imagickdraw, gettextdirection);
+PHP_METHOD(imagickdraw, settextdirection);
+#endif //#if MagickLibVersion >= 0x692
 
 /* Imagick Pixel iterator */
 PHP_METHOD(imagickpixeliterator, __construct);
 PHP_METHOD(imagickpixeliterator, newpixeliterator);
 PHP_METHOD(imagickpixeliterator, newpixelregioniterator);
+PHP_METHOD(imagickpixeliterator, getpixeliterator);
+PHP_METHOD(imagickpixeliterator, getpixelregioniterator);
 PHP_METHOD(imagickpixeliterator, getiteratorrow);
 PHP_METHOD(imagickpixeliterator, setiteratorrow);
 PHP_METHOD(imagickpixeliterator, getpreviousiteratorrow);
@@ -682,13 +1077,40 @@ PHP_METHOD(imagickpixel, __construct);
 PHP_METHOD(imagickpixel, setcolor);
 PHP_METHOD(imagickpixel, clear);
 PHP_METHOD(imagickpixel, destroy);
-PHP_METHOD(imagickpixel, issimilar);
+PHP_METHOD(imagickpixel, ispixelsimilarquantum);
+PHP_METHOD(imagickpixel, ispixelsimilar);
 PHP_METHOD(imagickpixel, getcolorvalue);
 PHP_METHOD(imagickpixel, setcolorvalue);
 PHP_METHOD(imagickpixel, getcolor);
+PHP_METHOD(imagickpixel, getcolorquantum);
 PHP_METHOD(imagickpixel, getcolorasstring);
 PHP_METHOD(imagickpixel, getcolorcount);
 PHP_METHOD(imagickpixel, setcolorcount);
 PHP_METHOD(imagickpixel, clone);
+#if MagickLibVersion >= 0x693
+PHP_METHOD(imagickpixel, setcolorfrompixel);
+#endif //#if MagickLibVersion >= 0x693
+
+#ifdef IMAGICK_WITH_KERNEL
+PHP_METHOD(imagickkernel, frommatrix);
+PHP_METHOD(imagickkernel, frombuiltin);
+PHP_METHOD(imagickkernel, addkernel);
+PHP_METHOD(imagickkernel, getmatrix);
+PHP_METHOD(imagickkernel, separate);
+PHP_METHOD(imagickkernel, scale);
+PHP_METHOD(imagickkernel, addunitykernel);
+#endif
+
+#if MagickLibVersion < 0x700
+#define KernelValueType double
+#else
+#define KernelValueType MagickRealType
+#endif
+
+#if MagickLibVersion >= 0x700
+#define IM_DEFAULT_CHANNEL UndefinedChannel
+#else
+#define IM_DEFAULT_CHANNEL DefaultChannels
+#endif
 
 #endif /* PHP_IMAGICK_DEFS_H */
